@@ -1,7 +1,7 @@
 /*
     webalizer - a web server log analysis program
 
-    Copyright (C) 1997-2000  Bradford L. Barrett (brad@mrunix.net)
+    Copyright (C) 1997-2008  Bradford L. Barrett
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,11 +19,6 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 
-    This software uses the gd graphics library, which is copyright by
-    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
-    see the documentation supplied with the library for additional
-    information and license terms, or visit www.boutell.com/gd/ for the
-    most recent version of the library and supporting documentation.
 */
 
 /*********************************************/
@@ -37,28 +32,22 @@
 #include <unistd.h>                           /* normal stuff             */
 #include <ctype.h>
 #include <sys/utsname.h>
-#include <sys/times.h>
-
-/* ensure getopt */
-#ifdef HAVE_GETOPT_H
-#include <getopt.h>
-#endif
 
 /* ensure sys/types */
 #ifndef _SYS_TYPES_H
 #include <sys/types.h>
 #endif
 
+/* need socket header? */
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+
 /* some systems need this */
 #ifdef HAVE_MATH_H
 #include <math.h>
 #endif
 
-/* SunOS 4.x Fix */
-#ifndef CLK_TCK
-#define CLK_TCK _SC_CLK_TCK
-#endif
-
 #include "webalizer.h"                        /* main header              */
 #include "lang.h"
 #include "linklist.h"
@@ -79,7 +68,7 @@ DNODEPTR new_dnode(char *);                   /* new DNS node             */
 void     update_entry(char *);                /* update entry/exit        */
 void     update_exit(char *);                 /* page totals              */
 
-u_long   hash(char *);                        /* hash function            */
+u_int64_t hash(char *);                       /* hash function            */
 
 /* local data */
 
@@ -152,16 +141,16 @@ HNODEPTR new_hnode(char *str)
 /* PUT_HNODE - insert/update host node       */
 /*********************************************/
 
-int put_hnode( char     *str,   /* Hostname  */
+int put_hnode( char      *str,  /* Hostname  */
                int       type,  /* obj type  */
-               u_long    count, /* hit count */
-               u_long    file,  /* File flag */
+               u_int64_t count, /* hit count */
+               u_int64_t file,  /* File flag */
                double    xfer,  /* xfer size */
-               u_long   *ctr,   /* counter   */
-               u_long    visit, /* visits    */
-               u_long    tstamp,/* timestamp */
-               char     *lasturl, /* lasturl */
-               HNODEPTR *htab)  /* ptr>next  */
+               u_int64_t *ctr,  /* counter   */
+               u_int64_t visit, /* visits    */
+               u_int64_t tstamp,/* timestamp */
+               char      *lasturl, /* lasturl */
+               HNODEPTR  *htab)  /* ptr>next  */
 {
    HNODEPTR cptr,nptr;
 
@@ -341,8 +330,8 @@ UNODEPTR new_unode(char *str)
 /* PUT_UNODE - insert/update URL node        */
 /*********************************************/
 
-int put_unode(char *str, int type, u_long count, double xfer,
-              u_long *ctr, u_long entry, u_long exit, UNODEPTR *htab)
+int put_unode(char *str, int type, u_int64_t count, double xfer,
+              u_int64_t *ctr, u_int64_t entry, u_int64_t exit, UNODEPTR *htab)
 {
    UNODEPTR cptr,nptr;
 
@@ -468,7 +457,8 @@ RNODEPTR new_rnode(char *str)
 /* PUT_RNODE - insert/update referrer node   */
 /*********************************************/
 
-int put_rnode(char *str, int type, u_long count, u_long *ctr, RNODEPTR *htab)
+int put_rnode(char *str, int type, u_int64_t count,
+              u_int64_t *ctr, RNODEPTR *htab)
 {
    RNODEPTR cptr,nptr;
 
@@ -587,7 +577,8 @@ ANODEPTR new_anode(char *str)
 /* PUT_ANODE - insert/update user agent node */
 /*********************************************/
 
-int put_anode(char *str, int type, u_long count, u_long *ctr, ANODEPTR *htab)
+int put_anode(char *str, int type, u_int64_t count,
+              u_int64_t *ctr, ANODEPTR *htab)
 {
    ANODEPTR cptr,nptr;
 
@@ -702,7 +693,7 @@ SNODEPTR new_snode(char *str)
 /* PUT_SNODE - insert/update search str node */
 /*********************************************/
 
-int put_snode(char *str, u_long count, SNODEPTR *htab)
+int put_snode(char *str, u_int64_t count, SNODEPTR *htab)
 {
    SNODEPTR cptr,nptr;
 
@@ -808,15 +799,15 @@ INODEPTR new_inode(char *str)
 /* PUT_INODE - insert/update ident node      */
 /*********************************************/
 
-int put_inode( char     *str,   /* ident str */
+int put_inode( char      *str,  /* ident str */
                int       type,  /* obj type  */
-               u_long    count, /* hit count */
-               u_long    file,  /* File flag */
+               u_int64_t count, /* hit count */
+               u_int64_t file,  /* File flag */
                double    xfer,  /* xfer size */
-               u_long   *ctr,   /* counter   */
-               u_long    visit, /* visits    */
-               u_long    tstamp,/* timestamp */
-               INODEPTR *htab)  /* hashtable */
+               u_int64_t *ctr,  /* counter   */
+               u_int64_t visit, /* visits    */
+               u_int64_t tstamp,/* timestamp */
+               INODEPTR  *htab) /* hashtable */
 {
    INODEPTR cptr,nptr;
 
@@ -976,7 +967,7 @@ DNODEPTR new_dnode(char *str)
 /* PUT_DNODE - insert/update dns host node   */
 /*********************************************/
 
-int put_dnode(char *str, struct in_addr *addr, DNODEPTR *htab)
+int put_dnode(char *str, void *addr, int len, DNODEPTR *htab)
 {
    DNODEPTR cptr,nptr;
 
@@ -988,8 +979,9 @@ int put_dnode(char *str, struct in_addr *addr, DNODEPTR *htab)
       /* not hashed */
       if ( (nptr=new_dnode(str)) != NULL)
       {
-         if (addr) memcpy(&nptr->addr, addr, sizeof(struct in_addr));
-            else   memset(&nptr->addr, 0, sizeof(struct in_addr));
+         if (addr) memcpy(&nptr->addr, addr, len);
+            else   memset(&nptr->addr, 0, sizeof(struct sockaddr_storage));
+         nptr->addrlen = len;
          nptr->next = NULL;
          htab[hash(str)] = nptr;
       }
@@ -1005,8 +997,9 @@ int put_dnode(char *str, struct in_addr *addr, DNODEPTR *htab)
       /* not found... */
       if ( (nptr = new_dnode(str)) != NULL)
       {
-         if (addr) memcpy(&nptr->addr, addr, sizeof(struct in_addr));
-            else   memset(&nptr->addr, 0, sizeof(struct in_addr));
+         if (addr) memcpy(&nptr->addr, addr, len);
+            else   memset(&nptr->addr, 0, sizeof(struct sockaddr_storage));
+         nptr->addrlen = len;
          nptr->next  = htab[hash(str)];
          htab[hash(str)]=nptr;
       }
@@ -1047,9 +1040,9 @@ void	del_dlist(DNODEPTR *htab)
 /* HASH - return hash value for string       */
 /*********************************************/
 
-u_long hash(char *str)
+u_int64_t hash(char *str)
 {
-   u_long hashval;
+   u_int64_t hashval;
    for (hashval = 0; *str != '\0'; str++)
       hashval = *str + 31 * hashval;
    return hashval % MAXHASH;
@@ -1133,7 +1126,7 @@ void update_exit(char *str)
 /* MONTH_UPDATE_EXIT  - eom exit page update */
 /*********************************************/
 
-void month_update_exit(u_long tstamp)
+void month_update_exit(u_int64_t tstamp)
 {
    HNODEPTR nptr;
    int i;
@@ -1157,10 +1150,10 @@ void month_update_exit(u_long tstamp)
 /* TOT_VISIT - calculate total visits        */
 /*********************************************/
 
-u_long tot_visit(HNODEPTR *list)
+u_int64_t tot_visit(HNODEPTR *list)
 {
    HNODEPTR   hptr;
-   u_long     tot=0;
+   u_int64_t  tot=0;
    int        i;
 
    for (i=0;i<MAXHASH;i++)
